<?php
// pdffile.php
// use PDF information to build a useful HTML interface
// and search form

require_once 'pdfportal.php';

// passed in
$src_pdf_fn= $_GET['pdf']; // path to pdf of interest
$qq= ''; // the search query, if any
if( array_key_exists('q', $_GET) ) {
  $qq= strtolower(urldecode($_GET['q']));
}

if( $qq ) { // parse this function only if we have a search query
  function
    output_hits( &$txt_fp,
		 &$hits_page_num,
		 &$current_page,
		 $qq,
		 $up_to_page,
		 $src_pdf_url )
    {
      $context_size= 40;
      $max_hits_per_page= 10;
      $hits_this_page= 0;

      while( ( $hits_page_num< $up_to_page ||
	       $up_to_page== -1 ) &&
	     ( $txt_line= fgets($txt_fp, 2048) )!== FALSE )
	{
	  $txt_line_len= strlen($txt_line);
	  if( $txt_line_len ) { // not empty
	    if( ord($txt_line)== 0x0c ) { // formfeed page break
	      ++$hits_page_num;

	      $current_page= 'Page '.$hits_page_num;
	      $hits_this_page= 0;
	    }
	    else { // 
	      $txt_line= strtolower($txt_line);
	      $qq_len= strlen($qq);
	      $offset= 0;
	      while( $offset< $txt_line_len &&
		     ($offset= strpos( $txt_line, $qq, $offset ))!== FALSE &&
		     $hits_this_page< $max_hits_per_page ) // output only so many excerpts per page
		{
		  $context_beg= ($offset- $context_size)< 0 ? 0 : $offset- $context_size;
		  $context_len= $context_size+ $qq_len+ $context_size;
		  if( $txt_line_len< $context_beg+ $context_len ) {
		    $context_len= $txt_line_len- $context_beg;
		  }

		  if( $current_page ) {
		    echo '<div class="hit_pg"><a href="'.$src_pdf_url.'#page='.$hits_page_num.'">'.
		      $current_page.'</a></div>';
		    $current_page= "";
		  }
		  echo '<div class="hit">...'.
		    str_replace( $qq, '<span class="hit_hl">'.$qq.'</span>', 
				 htmlentities( substr( $txt_line, $context_beg, $context_len ) ) ).
		    "...</div>\n";

		  $offset+= $qq_len;
		  ++$hits_this_page;
		}
	    }
	  }
	}
    }
}

function 
output_all_headings( $src_pdf_full_fn,
		     $src_pdf_url,
		     $pdf_bookmark_ar,
		     $pdf_txt_fn,
		     $qq )
{
  global $pdftotext_full_fn;
  $current_level= 0; // one less than is possible

  // create $pdf_txt_fn it if not present
  if( !file_exists($pdf_txt_fn) ) {
    if( $pdftotext_full_fn && file_exists($pdftotext_full_fn) ) {
      exec( "$pdftotext_full_fn $src_pdf_full_fn $pdf_txt_fn" );
    }
  }

  $txt_fp= @fopen( $pdf_txt_fn, "r" );
  $hits_page_num= 1;
  $current_page= "Page 1";

  foreach( $pdf_bookmark_ar as $bookmark ) {
    if( $qq && $txt_fp ) {
      output_hits( $txt_fp,
		   $hits_page_num,
		   $current_page,
		   $qq,
		   $bookmark['page'],
		   $src_pdf_url );
    }

    if( $bookmark['level']== $current_level ) {
      // same heading level
      echo "</li>\n<li>";
    }
    else if( $current_level< $bookmark['level'] ) {
      // increaed heading level
      while( $current_level< $bookmark['level'] ) {
	echo "\n<ul><li>";
	++$current_level;
      }
    }
    else {
      // decreased heading level
      echo '</li>';
      while( $bookmark['level']< $current_level ) {
	echo "</ul></li>\n";
	--$current_level;
      }
      echo '<li>';
    }

    echo 
      "<div class=\"hi-$current_level\"><a href=\"$src_pdf_url#page={$bookmark['page']}\">" .
      "{$bookmark['title']}</a></div>";

  }

  if( $qq && $txt_fp ) {
    // catch stragglers
    output_hits( $txt_fp,
		 $hits_page_num,
		 $current_page,
		 $qq,
		 -1,
		 $src_pdf_url );
  }

  // wind back down, if necessary
  while( 1< $current_level ) {
    echo '</li></ul>';
    --$current_level;
  }
  echo '</li></ul>';

  if( $txt_fp ) {
    fclose( $txt_fp );
  }
}

function
output_heading_parent_line( $src_pdf_url,
			    $pdf_bookmark_ar,
			    $bookmark_index )
{
  $bookmark= $pdf_bookmark_ar[$bookmark_index];

  $ii= $bookmark_index- 1;
  for( ; 0<= $ii; --$ii ) {
    if( $pdf_bookmark_ar[$ii]['level']< $bookmark['level'] ) {
      output_heading_parent_line( $src_pdf_url,
				  $pdf_bookmark_ar,
				  $ii );
      break;
    }
  }

  if( 0<= $ii ) {
    echo ' &gt; ';
  }
  echo 
    "<span class=\"hl-{$bookmark['level']}\"><a href=\"$src_pdf_url#page={$bookmark['page']}\">" .
    "{$bookmark['title']}</a></span>\n";
}

function
output_heading_parent_tree( $src_pdf_url,
			    $pdf_bookmark_ar,
			    $bookmark_index,
			    &$num_parents )
{
  $bookmark= $pdf_bookmark_ar[$bookmark_index];
  $num_parents_begin= $num_parents;
  ++$num_parents;

  $ii= $bookmark_index- 1;
  for( ; 0<= $ii; --$ii ) {
    if( $pdf_bookmark_ar[$ii]['level']< $bookmark['level'] ) {
      output_heading_parent_tree( $src_pdf_url,
				  $pdf_bookmark_ar,
				  $ii,
				  $num_parents );
      break;
    }
  }

  echo '<ul><li>';
  echo 
    "<span class=\"ht-{$bookmark['level']}\"><a href=\"$src_pdf_url#page={$bookmark['page']}\">" .
    "{$bookmark['title']}</a></span>\n";

  if( $num_parents_begin== 0 ) { // we were the first called; clean up
    for( $ii= 0; $ii< $num_parents; ++$ii ) {
      echo '</li></ul>';
    }
  }
}

// our own url
$pdffile_url= 'http://'.$_SERVER['HTTP_HOST'].$_SERVER['PHP_SELF'];

// our pdf of interest
$src_pdf_full_fn= $_SERVER['DOCUMENT_ROOT'].$src_pdf_fn;
$src_pdf_url= 'http://'.$_SERVER['HTTP_HOST'].$src_pdf_fn;

// search 
$pdf_txt_fn= $src_pdf_full_fn.'.txt';

// pdf info and bookmarks
$pdf_data_ar= array();
$pdf_info_ar= array();
$pdf_bookmark_ar= array();
//
load_pdf_info( $src_pdf_full_fn,
	       $pdf_data_ar,
	       $pdf_info_ar,
	       $pdf_bookmark_ar,
	       FALSE );

// document title
if( !array_key_exists('Title', $pdf_info_ar) ||
    !$pdf_info_ar['Title'] )
{
  $pdf_info_ar['Title']= basename($src_pdf_fn);
}

?>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
<style type="text/css">
<!--
div.metakey { font-family: sans-serif; font-size: 90% }
div.metadata { font-family: sans-serif; font-size: 90%; font-weight: bold }
div.editinfo, div.dirlink { font-family: sans-serif; font-size: 80% }
div.hit_pg { font-family: sans-serif; font-size: 70% }
div.hit { font-family: sans-serif; font-size: 90% }
span.hit_hl { background-color: yellow }
div.hi-1 { font-family: serif; margin-top: 1ex }
div.hi-2 { font-family: sans-serif; font-size: 90%; margin-top: 0.5ex }
div.hi-3 { font-family: sans-serif; font-size: 90%; margin-top: 0.5ex }
div.hi-4 { font-family: sans-serif; font-size: 90%; margin-top: 0.5ex }
div.hi-5 { font-family: sans-serif; font-size: 90%; margin-top: 0.5ex }
div.hi-6 { font-family: sans-serif; font-size: 90%; margin-top: 0.5ex }
div.hi-7 { font-family: sans-serif; font-size: 90%; margin-top: 0.5ex }
div.hi-8 { font-family: sans-serif; font-size: 90%; margin-top: 0.5ex }
-->
</style>
<title><?php echo $pdf_info_ar['Title'] ?></title>
</head>
<body>

<center><table border=0 width="100%"><tr>
<td align="center" valign="middle" width="15%" bgcolor="white">
<?php // an icon
if( $pdfportal_icon_url ) {
  echo '<img src="'.$pdfportal_icon_url.'" alt="Icon">';
}
?>
</td>
<td>&nbsp;</td>
<td align="left" bgcolor="#ddd">

<table border=0>
<tr valign="top">
<td align="right"><div class="metakey">Document Title:</div></td>
<td align="left"><div class="metadata"><?php if(array_key_exists('Title',$pdf_info_ar)){ echo $pdf_info_ar['Title']; } ?></div></td></tr>
<tr valign="top">
<td align="right"><div class="metakey">Authors:</div></td>
<td align="left"><div class="metadata"><?php if(array_key_exists('Author',$pdf_info_ar)){ echo $pdf_info_ar['Author']; } ?></div></td></tr>
<tr valign="top">
<td align="right"><div class="metakey">Date Created:</div></td>
<td align="left"><div class="metadata"><?php if(array_key_exists('CreationDate',$pdf_info_ar)){ echo readable_pdf_date($pdf_info_ar['CreationDate']); } ?></div></td></tr>
<tr valign="top">
<td align="right"><div class="metakey">Last Modified:</div></td>
<td align="left"><div class="metadata"><?php if(array_key_exists('ModDate',$pdf_info_ar)){ echo readable_pdf_date($pdf_info_ar['ModDate']); } ?></div></td></tr>
<tr valign="top">
<td align="right"><div class="metakey">Number of Pages:</div></td>
<td align="left"><div class="metadata"><?php if(array_key_exists('NumberOfPages',$pdf_data_ar)){ echo $pdf_data_ar['NumberOfPages']; } ?></div></td></tr>
</table>

</td>
<td valign="bottom"><center>
<?php
// pdf search; either the PDF text dump must be available, or pdftotext
// must be available to create the text dump on demand
if( file_exists($pdf_txt_fn) || $pdftotext_full_fn && file_exists($pdftotext_full_fn) ) {
echo 
'<div><form method="get" action="'.$pdffile_url.'">
<input type="text" name="q" value="'.$qq.'">
<input type="hidden" name="pdf" value="'.$src_pdf_fn.'"><br>
<input type="submit" value="Search Document">
</form></div>';
}

// editinfo
$editinfo_fn= dirname($_SERVER['PHP_SELF']).'/editinfo.php';
$editinfo_full_fn= $_SERVER['DOCUMENT_ROOT'].$editinfo_fn;
$editinfo_url= 'http://'.$_SERVER['HTTP_HOST'].$editinfo_fn;
//
if( $editinfo_full_fn && file_exists($editinfo_full_fn) ) { // include 'edit info' link
  echo '<div class="editinfo"><a href="'.$editinfo_url.'?pdf='.$src_pdf_fn.'">Edit Document Data >></a></div>';
}

// document directory link
$pdfdir_fn= dirname($_SERVER['PHP_SELF']).'/pdfdir.php';
$pdfdir_full_fn= $_SERVER['DOCUMENT_ROOT'].$pdfdir_fn;
$pdfdir_url= 'http://'.$_SERVER['HTTP_HOST'].$pdfdir_fn;
//
if( $pdfdir_full_fn && file_exists($pdfdir_full_fn) ) { // include 'document directory' link
  echo '<div class="dirlink"><a href="'.$pdfdir_url.'?dir='.dirname($src_pdf_fn).'/">Document Directory >></a></div>';
}

?>
</center></td>
</tr></table></center>
<hr>
<table border=0 width="100%"><tr>
<td align="center" valign="top" width="15%" bgcolor="#ccc">&nbsp;<br>
<?php 
// a document icon, if present
if( is_readable($src_pdf_full_fn.'.png') ) {
  echo '<img src="'.$src_pdf_url.'.png'.'" alt="Icon">';
}
else if( is_readable($src_pdf_full_fn.'.gif') ) {
  echo '<img src="'.$src_pdf_url.'.gif'.'" alt="Icon">';
}
else if( is_readable($src_pdf_full_fn.'.jpg') ) {
  echo '<img src="'.$src_pdf_url.'.jpg'.'" alt="Icon">';
}
?>
</td>
<td align="left">
<?php
output_all_headings( $src_pdf_full_fn,
		     $src_pdf_url,
		     $pdf_bookmark_ar,
		     $pdf_txt_fn,
		     $qq );
?>
</td>
</tr></table>
<hr>
<center><div style="font-family: sans-serif; font-size: 80%">Powered by <a href="http://www.pdfhacks.com">PDF Hacks</a></div></center>
</body></html>
