<?php
// pdfportal.php
//
// load_pdf_info
// output_all_headings
// output_heading_parent_line
// output_heading_parent_tree
// parse_pdf_date
// readable_pdf_date

require_once 'pdfportal.config.php';


function 
load_pdf_info( $src_pdf_full_fn,
	       &$pdf_data_ar,
	       &$pdf_info_ar,
	       &$pdf_bookmark_ar,
	       $pdf_info_only_b )
{
  global $pdftk_full_fn;

  if( is_readable($src_pdf_full_fn) ) {
    $pdf_info_fn= $src_pdf_full_fn.'.info';

    if( !file_exists($pdf_info_fn) ) {
      if( $pdftk_full_fn && file_exists($pdftk_full_fn) ) {
	// create PDF info file
	exec( "$pdftk_full_fn $src_pdf_full_fn dump_data output $pdf_info_fn" );
      }
    }

    // parse info file
    $pdf_info_fp= @fopen( $pdf_info_fn, "r" );
    if( $pdf_info_fp ) {
      $temp1= "";
      $temp2= "";

      while( ($info_line= fgets($pdf_info_fp, 2048))!== FALSE ) {
	$info_line= rtrim( $info_line ); // drop any term. whitespace, like \n
	$data_pos= strpos( $info_line, ':' );
	$name= substr( $info_line, 0, $data_pos+ 1 );

	switch( $name ) {
	    
	  // pdf_data_ar items
	case 'NumberOfPages:':
	  $pdf_data_ar['NumberOfPages']= (int)substr( $info_line, $data_pos+ 2 );
	  break;
	case 'PdfID0:':
	  $pdf_data_ar['PdfID0']= substr( $info_line, $data_pos+ 2 );
	  break;
	case 'PdfID1:':
	  $pdf_data_ar['PdfID1']= substr( $info_line, $data_pos+ 2 );
	  break;

	  // pdf_info_ar items
	case 'InfoKey:':
	  $temp1= substr( $info_line, $data_pos+ 2 );
	  break;
	case 'InfoValue:':
	  $pdf_info_ar[$temp1]= substr( $info_line, $data_pos+ 2 );
	  $temp1= "";
	  break;

	  // pdf_bookmark_ar items
	case 'BookmarkTitle:':
	  if( !$pdf_info_only_b ) {
	    $temp1= substr( $info_line, $data_pos+ 2 );
	  }
	  break;
	case 'BookmarkLevel:':
	  if( !$pdf_info_only_b ) {
	    $temp2= substr( $info_line, $data_pos+ 2 );
	  }
	  break;
	case 'BookmarkPageNumber:':
	  if( !$pdf_info_only_b ) {
	    // pack our new bookmark into the array
	    $page_num= substr( $info_line, $data_pos+ 2 );
	    $pdf_bookmark_ar[]= 
	      array( "title" => $temp1,
		     "level" => (int)$temp2,
		     "page" => (int)$page_num );

	    $temp1= "";
	    $temp2= "";
	  }
	  break;
	}
      }

      fclose( $pdf_info_fp );
    } // end: if( $pdf_info_fp )
  } // end: is_readable( $src_pdf_full_fn )
}

function
save_pdf_info( $src_pdf_full_fn,
	       $pdf_data_ar,
	       $pdf_info_ar,
	       $pdf_bookmark_ar )
{
  // save the info into a temp file and then rename it
  $pdf_info_temp_fn= tempnam( dirname($src_pdf_full_fn), "info" );
  $pdf_info_full_fn= $src_pdf_full_fn.'.info';

  if( $pdf_info_temp_fn ) {
    $pdf_info_temp_fp= @fopen( $pdf_info_temp_fn, "wb" );
    if( $pdf_info_temp_fp ) {
      $success_b= TRUE;

      // output info array
      foreach( $pdf_info_ar as $key => $value ) {
	$success_b= 
	  fwrite( $pdf_info_temp_fp, 
		  'InfoKey: '.$key."\n".
		  'InfoValue: '.$value."\n" );
	if( !$success_b )
	  break;
      }

      if( $success_b ) {
	// output data array
	foreach( $pdf_data_ar as $key => $value ) {
	  $success_b=
	    fwrite( $pdf_info_temp_fp,
		    $key.': '.$value."\n" );
	  if( !$success_b )
	    break;
	}
      }

      if( $success_b ) {
	// output bookmark array
	foreach( $pdf_bookmark_ar as $bookmark ) {
	  $success_b= 
	    fwrite( $pdf_info_temp_fp, 
		    'BookmarkTitle: '.$bookmark['title']."\n".
		    'BookmarkLevel: '.$bookmark['level']."\n".
		    'BookmarkPageNumber: '.$bookmark['page']."\n" );
	  if( !$success_b )
	    break;
	}
      }
      
      fclose( $pdf_info_temp_fp );

      if( $success_b ) {
	// rename this temp info file to replace the standard info file
	unlink( $pdf_info_full_fn );
	rename( $pdf_info_temp_fn, $pdf_info_full_fn );
      }
    }
  }
}

// PDF dates are described in the PDF Reference;
// we are only using the first 12 chars, if present;
// the remainder is time zone information;
function
parse_pdf_date( $pdf_date_string,
		&$date_ar )
{
  $num_fields= 0;
  $months= array( 1 => "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" );

  if( $pdf_date_string{0}== 'D' ) {
    // drop initial "D:", if present
    $pdf_date_string= substr( $pdf_date_string, 2 );
  }

  // defaults
  $date_ar['month']= 1;
  $date_ar['day']= 1;
  
  $date_len= min( strlen($pdf_date_string), 12 );
  if( 4<= $date_len ) {
    // grab year
    $date_ar['year']= (int)substr( $pdf_date_string, 0, 4 );
    $num_fields= 3;
  }
  if( 6<= $date_len ) {
    $date_ar['month']= (int)substr( $pdf_date_string, 4, 2 );
    $date_ar['month_string']= $months[$date_ar['month']];
  }
  if( 8<= $date_len ) {
    $date_ar['day']= (int)substr( $pdf_date_string, 6, 2 );
  }

  if( 10<= $date_len ) {
    $date_ar['hour']= (int)substr( $pdf_date_string, 8, 2 );
    ++$num_fields;
  }
  if( 12<= $date_len ) {
    // keep as a string, to preserve init. zero
    $date_ar['minute']= substr( $pdf_date_string, 10, 2 );
    ++$num_fields;
  }

  return $num_fields;
}

function 
readable_pdf_date( $pdf_date_string )
{
  $date= "";
  $date_ar= array();
  $num_fields= parse_pdf_date( $pdf_date_string, $date_ar );

  if( $num_fields ) {
    // we have year, month and day
    $date= $date_ar['month_string'].' '.$date_ar['day'].', '.$date_ar['year'];

    if( $num_fields== 5 ) {
      // we also have hour and minute
      $date= $date. ' - '.$date_ar['hour'].':'.$date_ar['minute'];
    }
  }

  return $date;
}

?>